//
// Created by robin on 1/22/18.
//

#include "dns-base/RecordData/QDnsRecordDataAAAA.h"

namespace QDns
{
    namespace Base
    {
        QDnsRecordDataAAAA::QDnsRecordDataAAAA()
                : QDnsRecordData()
        {
        }

        QDnsRecordDataAAAA::QDnsRecordDataAAAA(const QByteArray &packetData, quint32 pos)
                : QDnsRecordDataAAAA()
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            parse(packetData, dataStream);
        }

        QDnsRecordDataAAAA::QDnsRecordDataAAAA(const QByteArray &packetData, QDataStream &dataStream)
                : QDnsRecordDataAAAA()
        {
            parse(packetData, dataStream);
        }

        void QDnsRecordDataAAAA::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            if (dataStream.device()->bytesAvailable() < 16)
            {
                m_isValid = false;
            }
            else
            {
                Q_IPV6ADDR ip;
                for (int i = 0; i < 16; ++i)
                {
                    quint8 byte;
                    dataStream >> byte;
                    ip[i] = byte;
                }
                m_ip.setAddress(ip);
            }
        }

        QByteArray QDnsRecordDataAAAA::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsRecordDataAAAA::serialize(QDataStream &dataStream) const
        {
            Q_IPV6ADDR ip = m_ip.toIPv6Address();
            for (int i = 0; i < 16; ++i)
            {
                dataStream << ip[i];
            }
        }

        QDnsRecordData *QDnsRecordDataAAAA::clone() const
        {
            auto data = new QDnsRecordDataAAAA();
            data->m_isValid = m_isValid;
            data->m_ip = m_ip;
            return data;
        }

        const QHostAddress &QDnsRecordDataAAAA::getIp() const
        {
            return m_ip;
        }

        void QDnsRecordDataAAAA::setIp(const QHostAddress &ip)
        {
            m_ip = ip;
        }

        QString QDnsRecordDataAAAA::toString() const
        {
            return getIp().toString();
        }
    }
}