//
// Created by robin on 1/14/18.
//

#include <gtest/gtest.h>
#include <QByteArray>
#include <QString>
#include <QtNetwork/QHostAddress>
#include "dns-base/RecordData/QDnsRecordDataMX.h"

struct DnsRDataMXTestParams
{
    bool isValid;
    const char* hexData;
    quint32 pos;

    quint16 preference;
    QString mx;
};

class DnsRDataMXTest : public ::testing::TestWithParam<DnsRDataMXTestParams>
{
};

TEST_P(DnsRDataMXTest, parseAndGetters)
{
    auto params = GetParam();
    auto data = QByteArray::fromHex(params.hexData);
    auto rdata = dynamic_cast<QDns::Base::QDnsRecordDataMX*>(QDns::Base::QDnsRecordDataMX(data, params.pos).clone());
    ASSERT_NE(rdata, nullptr);
    ASSERT_EQ(rdata->isValid(), params.isValid);
    
    if (rdata->isValid())
    {
        ASSERT_EQ(params.preference, rdata->getPreference());
        ASSERT_EQ(params.mx, rdata->getMx());

        ASSERT_EQ(rdata->serialize().toHex(), QString(params.hexData).remove(" "));
    }
    delete rdata;
}

INSTANTIATE_TEST_CASE_P(DnsRDataMXTestInst,
                        DnsRDataMXTest,
                        ::testing::Values(
                                DnsRDataMXTestParams {false, "", 0},
                                DnsRDataMXTestParams {false, "00", 0},
                                DnsRDataMXTestParams {true, "003204616c7434056173706d78016c06676f6f676c6503636f6d00", 0, 50, "alt4.aspmx.l.google.com"}
                        ));